package sun.net.util;

import java.net.URL;
import java.util.Arrays;

public class IPAddressUtil
{
    private static final int INADDR4SZ = 4;
    private static final int INADDR16SZ = 16;
    private static final int INT16SZ = 2;
    private static final long L_IPV6_DELIMS = 0L;
    private static final long H_IPV6_DELIMS = 671088640L;
    private static final long L_GEN_DELIMS = -8935000888854970368L;
    private static final long H_GEN_DELIMS = 671088641L;
    private static final long L_AUTH_DELIMS = 288230376151711744L;
    private static final long H_AUTH_DELIMS = 671088641L;
    private static final long L_COLON = 288230376151711744L;
    private static final long H_COLON = 0L;
    private static final long L_SLASH = 140737488355328L;
    private static final long H_SLASH = 0L;
    private static final long L_BACKSLASH = 0L;
    private static final long H_BACKSLASH = 268435456L;
    private static final long L_NON_PRINTABLE = 4294967295L;
    private static final long H_NON_PRINTABLE = Long.MIN_VALUE;
    private static final long L_EXCLUDE = -8935000884560003073L;
    private static final long H_EXCLUDE = -9223372035915251711L;
    private static final char[] OTHERS;
    
    public IPAddressUtil() {
    }
    
    public static byte[] textToNumericFormatV4(String s) {
        byte[] array = new byte[4];
        long n = 0L;
        int n2 = 0;
        int n3 = 1;
        int length = s.length();
        if (length == 0 || length > 15) {
            return null;
        }
        for (int i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            if (char1 == '.') {
                if (n3 != 0 || n < 0L || n > 255L || n2 == 3) {
                    return null;
                }
                array[n2++] = (byte)(int)(n & 0xFFL);
                n = 0L;
                n3 = 1;
            }
            else {
                int digit = Character.digit((char)char1, 10);
                if (digit < 0) {
                    return null;
                }
                n = n * 10L + (long)digit;
                n3 = 0;
            }
        }
        if (n3 != 0 || n < 0L || n >= 1L << (4 - n2) * 8) {
            return null;
        }
        switch (n2) {
            case 0: {
                array[0] = (byte)(int)(n >> 24 & 0xFFL);
            }
            case 1: {
                array[1] = (byte)(int)(n >> 16 & 0xFFL);
            }
            case 2: {
                array[2] = (byte)(int)(n >> 8 & 0xFFL);
            }
            case 3: {
                array[3] = (byte)(int)(n >> 0 & 0xFFL);
                break;
            }
        }
        return array;
    }
    
    public static byte[] textToNumericFormatV6(String s) {
        if (s.length() < 2) {
            return null;
        }
        char[] charArray = s.toCharArray();
        byte[] array = new byte[16];
        int length = charArray.length;
        int index = s.indexOf("%");
        if (index == length - 1) {
            return null;
        }
        if (index != -1) {
            length = index;
        }
        int n = -1;
        int i = 0;
        int n2 = 0;
        if (charArray[i] == ':' && charArray[++i] != ':') {
            return null;
        }
        int n3 = 0;
        int n4 = 0;
        while (i < length) {
            int ch = charArray[i++];
            int digit = Character.digit((char)ch, 16);
            if (digit != -1) {
                n4 = (n4 << 4 | digit);
                if (n4 > 65535) {
                    return null;
                }
                n3 = 1;
            }
            else if (ch == ':') {
                if (n3 == 0) {
                    if (n != -1) {
                        return null;
                    }
                    n = n2;
                }
                else {
                    if (i == length) {
                        return null;
                    }
                    if (n2 + 2 > 16) {
                        return null;
                    }
                    array[n2++] = (byte)(n4 >> 8 & 0xFF);
                    array[n2++] = (byte)(n4 & 0xFF);
                    n3 = 0;
                    n4 = 0;
                }
            }
            else if (ch == '.' && n2 + 4 <= 16) {
                String substring = s.substring(i, length);
                int n5 = 0;
                for (int index2 = 0; (index2 = substring.indexOf(46, index2)) != -1; ++index2) {
                    ++n5;
                }
                if (n5 != 3) {
                    return null;
                }
                byte[] textToNumericFormatV4 = textToNumericFormatV4(substring);
                if (textToNumericFormatV4 == null) {
                    return null;
                }
                for (int j = 0; j < 4; ++j) {
                    array[n2++] = textToNumericFormatV4[j];
                }
                n3 = 0;
                break;
            }
            else {
                return null;
            }
        }
        if (n3 != 0) {
            if (n2 + 2 > 16) {
                return null;
            }
            array[n2++] = (byte)(n4 >> 8 & 0xFF);
            array[n2++] = (byte)(n4 & 0xFF);
        }
        if (n != -1) {
            int n6 = n2 - n;
            if (n2 == 16) {
                return null;
            }
            for (int k = 1; k <= n6; ++k) {
                array[16 - k] = array[n + n6 - k];
                array[n + n6 - k] = 0;
            }
            n2 = 16;
        }
        if (n2 != 16) {
            return null;
        }
        byte[] convertFromIPv4MappedAddress = convertFromIPv4MappedAddress(array);
        if (convertFromIPv4MappedAddress != null) {
            return convertFromIPv4MappedAddress;
        }
        return array;
    }
    
    public static boolean isIPv4LiteralAddress(String s) {
        return textToNumericFormatV4(s) != null;
    }
    
    public static boolean isIPv6LiteralAddress(String s) {
        return textToNumericFormatV6(s) != null;
    }
    
    public static byte[] convertFromIPv4MappedAddress(byte[] array) {
        if (isIPv4MappedAddress(array)) {
            byte[] array2 = new byte[4];
            System.arraycopy(array, 12, array2, 0, 4);
            return array2;
        }
        return null;
    }
    
    private static boolean isIPv4MappedAddress(byte[] array) {
        return array.length >= 16 && (array[0] == 0 && array[1] == 0 && array[2] == 0 && array[3] == 0 && array[4] == 0 && array[5] == 0 && array[6] == 0 && array[7] == 0 && array[8] == 0 && array[9] == 0 && array[10] == -1 && array[11] == -1);
    }
    
    public static boolean match(char c, long n, long n2) {
        if (c < '@') {
            return (1L << c & n) != 0x0L;
        }
        return c < '\u0080' && (1L << c - 64 & n2) != 0x0L;
    }
    
    public static int scan(String s, long n, long n2) {
        int index = -1;
        int length;
        if (s == null || (length = s.length()) == 0) {
            return -1;
        }
        boolean match = false;
        while (++index < length && (match = match(s.charAt(index), n, n2)) == 0) {}
        if (match != 0) {
            return index;
        }
        return -1;
    }
    
    public static int scan(String s, long n, long n2, char[] a) {
        int index = -1;
        int length;
        if (s == null || (length = s.length()) == 0) {
            return -1;
        }
        boolean match = false;
        int n3 = a[0];
        int char1;
        while (++index < length && (match = match((char)(char1 = s.charAt(index)), n, n2)) == 0) {
            if (char1 < n3 || Arrays.binarySearch(a, (char)char1) <= -1) {
                continue;
            }
            match = true;
            break;
        }
        if (match != 0) {
            return index;
        }
        return -1;
    }
    
    private static String describeChar(char c) {
        if (c < ' ' || c == '\u007f') {
            if (c == '\n') {
                return "LF";
            }
            if (c == '\r') {
                return "CR";
            }
            return "control char (code=" + (int)c + ")";
        }
        else {
            if (c == '\\') {
                return "'\\'";
            }
            return "'" + c + "'";
        }
    }
    
    private static String checkUserInfo(String s) {
        int scan = scan(s, -9223231260711714817L, -9223372035915251711L);
        if (scan >= 0) {
            return "Illegal character found in user-info: " + describeChar(s.charAt(scan));
        }
        return null;
    }
    
    private static String checkHost(String s) {
        if (s.startsWith("[") && s.endsWith("]")) {
            s = s.substring(1, s.length() - 1);
            if (isIPv6LiteralAddress(s)) {
                int index = s.indexOf(37);
                if (index >= 0) {
                    int scan2 = scan(s = s.substring(index), 4294967295L, -9223372036183687168L);
                    if (scan2 >= 0) {
                        return "Illegal character found in IPv6 scoped address: " + describeChar(s.charAt(scan2));
                    }
                }
                return null;
            }
            else {
                return "Unrecognized IPv6 address format";
            }
        }
        else {
            int scan = scan(s, -8935000884560003073L, -9223372035915251711L);
            if (scan >= 0) {
                return "Illegal character found in host: " + describeChar(s.charAt(scan));
            }
            return null;
        }
    }
    
    private static String checkAuth(String s) {
        int scan = scan(s, -9223231260711714817L, -9223372036586340352L);
        if (scan >= 0) {
            return "Illegal character found in authority: " + describeChar(s.charAt(scan));
        }
        return null;
    }
    
    public static String checkAuthority(URL url) {
        if (url == null) {
            return null;
        }
        String userInfo;
        String checkUserInfo;
        if ((checkUserInfo = checkUserInfo(userInfo = url.getUserInfo())) != null) {
            return checkUserInfo;
        }
        String host;
        String checkHost;
        if ((checkHost = checkHost(host = url.getHost())) != null) {
            return checkHost;
        }
        if (host == null && userInfo == null) {
            return checkAuth(url.getAuthority());
        }
        return null;
    }
    
    public static String checkExternalForm(URL url) {
        if (url == null) {
            return null;
        }
        String userInfo;
        int scan = scan(userInfo = url.getUserInfo(), 140741783322623L, Long.MIN_VALUE);
        if (scan >= 0) {
            return "Illegal character found in authority: " + describeChar(userInfo.charAt(scan));
        }
        String checkHostString;
        if ((checkHostString = checkHostString(url.getHost())) != null) {
            return checkHostString;
        }
        return null;
    }
    
    public static String checkHostString(String s) {
        if (s == null) {
            return null;
        }
        int scan = scan(s, 140741783322623L, Long.MIN_VALUE, IPAddressUtil.OTHERS);
        if (scan >= 0) {
            return "Illegal character found in host: " + describeChar(s.charAt(scan));
        }
        return null;
    }
    
    static {
        OTHERS = new char[] { '⁇', '⁈', '⁉', '℀', '℁', '℅', '℆', '⩴', '﹕', '﹖', '﹟', '﹫', '＃', '／', '：', '？', '＠' };
    }
}
